#include <octave/oct.h>
#include <algorithm>
#include <vector>
using namespace std;

DEFUN_DLD (accumarray_max, args, nargout,
	"Calculate  accumarray(i,v,[n,1],@max,-Inf)\n"
) {
	octave_value_list retval;

	// parse arguments
	if (args.length() != 3) {
		print_usage();
		error("Expected 3 arguments");
		return retval;
	}
	const Matrix is = args(0).matrix_value();
	const Matrix vs = args(1).matrix_value();
	int n = args(2).int_value();
	
	// check
	if (is.nelem() != vs.nelem()) {
		error("Arguments should have the same size");
		return retval;
	}
	
	// go
	ColumnVector out(n);
	out.fill(-INFINITY);
	
	for (int i = 0 ; i < is.nelem() ; ++i) {
		int j = is(i)-1;
		if (j < 0 || j >= n) {
			error("Index out of range");
			continue;
		}
		out(j) = max(out(j),vs(i));
	}
	
	retval(0) = out;
	return retval;
}
