#include <octave/oct.h>
#include <algorithm>
#include <vector>
#include <boost/static_assert.hpp>
using namespace std;

void go(SparseMatrix const& a, int* lbl, int l, int i) {
	for (int e = a.cidx(i); e < a.cidx(i+1); ++e) {
		int j = a.ridx(e);
		if (lbl[j] == 0) {
			lbl[j] = l;
			go(a,lbl,l,j);
		}
	}
}

DEFUN_DLD (connected_components, args, nargout,
	"Find connected components given a sparse adjacency matrix A.\n"
	"The connected components will be labled with different integers\n"
	"Not a terribly efficient implementation\n"
) {
	octave_value_list retval;

	// parse arguments
	if (args.length() != 1) {
		print_usage();
		error("Expected 1 argument");
		return retval;
	}
	const SparseMatrix a = args(0).sparse_matrix_value();
	// sizes
	int n = a.dims()(0);
	// check
	if (a.dims()(0) != a.dims()(1)) {
		error("Argument should be a square matrix");
		return retval;
	}
	
	// memory
	int32NDArray labels(dim_vector(n,1));
	int* labels_ = reinterpret_cast<int*>(labels.fortran_vec());
	BOOST_STATIC_ASSERT(sizeof(labels(0)) == sizeof(int));
	std::fill(labels_,labels_+n,0);
	
	int label = 1;
	for (int i = 0 ; i < n ; ++i) {
		if (labels_[i] == 0) {
			labels_[i] = label;
			go(a,labels_,label,i);
			label++;
		}
	}
	
	retval(0) = labels;
	return retval;
}
