function [Y,same,diff] = edge_labels(A,c)
	% Construct labels to use in a classifier for edges in A
	% Edges within a community get label +1,
	% Edges between different communities get label -1
	% other edges get label 0
	%
	% Parameters:
	%   c(i) >= 0  means: known community label for node i
	%   c(i) == -1 means: unknown community label for node i
	%   c(i) <  -1 means: semi-unknown commmunity label for node i,
	%                     it is different from other (non -1) communities,
	%                     but might be subdivided further
	
	% Prepare data: Y
	[i,j] = find(A);
	same = (c(i) == c(j) & c(i) >= 0);
	diff = (c(i) ~= c(j) & c(i) ~= -1 & c(j) ~= -1);
	Y = same - diff; % postive - negative
	Y = vec(Y);
end
