
function out = function_to_string(x, limit)
	% Convert a value, in particular a function, to a string
	% Gives a better result for anonymous functions compared to func2str
	%
	% Usage: function_to_string(@some_function_handle)
	
	if isa(x,'function_handle')
		info = functions(x);
		out = info.function;
		
		if isequal(info.type,'anonymous')
			if isfield(info,'workspace') && isfield(info.workspace,'_name_')
				% manually renamed function
				out = info.workspace._name_;
				return;
			end
			
			if isfield(info,'workspace')
				% subsittude workspace:
				% replace arguments by their bound values
				ns = fieldnames(info.workspace);
				for i=1:numel(ns)
					name = ns{i};
					if name(1)=='.', continue; end
					value = getfield(info.workspace, ns{i});
					value = function_to_string(value);
					out = regexprep(out,['\b',name,'\b'],value);
				end
			end
			
			% Try to clean up the result a bit
			out = regexprep(out,'(, )?varargin {:}','');
			out = regexprep(out,' \(','(');
			out = regexprep(out,' \{','{');
			out = regexprep(out,'^@\(\)\s*(\S*)_\(','$1(');
			out = regexprep(out,'([,;)]) ','$1');
			
			if nargin > 1 && length(out) > limit
				% shorten some more
				out = regexprep(out,'([,;)]) ','$1');
			end
			% Drop function arguments
			if (nargin > 1 && length(out) > limit) || 1
				[x1,x2,x3,x4,arg] = regexp(out,'^@\(([^)]*)\)');
				if numel(arg)
					arg = arg{1}{1};
					out = regexprep(out,'^@\(([^)]*)\)\s*','');
					out = regexprep(out,['\b',arg,'\b'],'_');
				end
			end
			if nargin > 1 && nargin > 1 && length(out) > limit
				% abreviate function names to 3 chars
				if 0
					mylength = @(x) length(regexprep(x,'…','.'));
					if mylength(out) > limit
						out = regexprep(out,'((^|[-+*.{(),_])[a-z]{4})[a-z]{3,}([a-z]{2})','$1…$3');
					end
					if mylength(out) > limit
						out = regexprep(out,'((^|[-+*.{(),_])[a-z]{4})([a-z]*…[a-z]*|[a-z]{2,})','$1…');
					end
					if mylength(out) > limit
						out = regexprep(out,'((^|[-+*.{(),_])[a-z]{3})([a-z]{1,}…[a-z]*|[a-z]{2,})','$1…');
					end
					if mylength(out) > limit
						out = regexprep(out,'((^|[- +*.{(),_])[a-z]{2})([a-z]{1,}…[a-z]*|[a-z]{2,})','$1…');
					end
				end
				if length(out) > limit
					out = regexprep(out,'((^|[-+*.{(),_])[a-z]{4})([a-z]*%[a-z]*|[a-z]{2,})','$1%');
				end
				if length(out) > limit
					out = regexprep(out,'((^|[-+*.{(),_])[a-z]{3})([a-z]{1,}%[a-z]*|[a-z]{2,})','$1%');
				end
				if length(out) > limit
					out = regexprep(out,'((^|[- +*.{(),_])[a-z]{2})([a-z]{1,}%[a-z]*|[a-z]{2,})','$1%');
				end
				out  = regexprep(out,'%','');
			end
		end
		
	elseif ischar(x)
		out = x;
		
	elseif iscell(x)
		out = '{';
		for i=1:numel(x)
			if i>1, out = [out,',']; end
			value = function_to_string(x{i});
			out = [out,value];
		end
		out = [out,'}'];
	elseif numel(x) > 2
		out = '...';
	elseif isscalar(x) && isnumeric(x)
		out = sprintf("%g",x);
	else
		out = mat2str(x);
	end
end
