
function [A,labels] = load_dataset(name)
	% Load a dataset by name
	% Usage:
	%   [A,labels] = load_dataset(name)
	% Possible names are:
	%   'zachary', 'football', 'polbooks', 'polblogs'
	%      For real-world networks
	%   'Small1000 <mu>','Big1000 <mu>','Small5000 <mu>','Big5000 <mu>','GN <mu>'
	%      For a LFR benchmark graph. Will be different on each run.
	%      <mu> is the mixing parameter between 0 and 1.
	%      For example 'Small1000 0.6'
	
	data_dir = file_in_path(path,'football.net');
	if isempty(data_dir)
		basepath = '../data/';
	else
		basepath = data_dir(1:end-12);
	end
	
	if isequal(name,'zachary') || isequal(name,'karate')
		A = load_pajek([basepath 'zachary.net'], 0);
		%labels = [1 1 1 1 1 1 1 1 2 2 1 1 1 1 2 2 1 1 2 1 2 1 2 2 2 2 2 2 2 2 2 2 2 2]; % what is the ground truth here?
		faction = [1 1 1 1 1 1 1 1 2 3 1 1 1 1 2 2 4 1 5 1 2 1 2 2 2 2 2 2 2 2 2 2 2 2]';
		labels  = [1 1 1 1 1 1 1 1 1 2 1 1 1 1 2 2 1 1 2 1 2 1 2 2 2 2 2 2 2 2 2 2 2 2]';
	elseif isequal(name,'football')
		A = load_pajek([basepath 'football.net'], 0);
		labels = load([basepath 'football_names.txt']);
		labels = [labels(2:end);labels(1)]; % error in file, 0 became last
	elseif isequal(name,'yeast') || isequal(name,'Yeast') || isequal(name,'YeastL')
		A = load_pajek([basepath 'YeastL.net'], 0);
		labels = load([basepath 'Yeast.clu']);
	elseif isequal(name,'polbooks')
		[A,c] = load_pajek([basepath 'polbooks.net'], 0);
		labels = cellfun(@(x)x + 0,c);
	elseif isequal(name,'polblogs')
		[A,c] = load_pajek([basepath 'polblogs.net'], 0);
		labels = cellfun(@str2num,c);
	elseif isequal(name,'example1')
		% First example from the paper
		A =      [0 1 1 1 0  0 0 0 0;
		          1 0 1 0 1  0 0 0 0;
		          1 1 0 1 1  0 0 0 0;
		          1 0 1 0 1  0 0 0 0;
		          0 1 1 1 0  1 1 0 0;
		          0 0 0 0 1  0 1 1 1;
		          0 0 0 0 1  1 0 1 1;
		          0 0 0 0 0  1 1 0 1;
		          0 0 0 0 0  1 1 1 0];
		labels = [1 1 1 1 1  2 2 2 2];
	elseif isequal(name,'example2')
		% Second example from the paper
		A =      [0 1 1 1 1  0 0 0 0 0;
		          1 0 1 1 1  0 0 0 0 0;
		          1 1 0 1 1  0 0 0 0 0;
		          1 1 1 0 1  0 0 0 0 0;
		          1 1 1 1 0  1 0 0 0 0;
		          0 0 0 0 1  0 1 1 1 1;
		          0 0 0 0 0  1 0 1 1 1;
		          0 0 0 0 0  1 1 0 1 1;
		          0 0 0 0 0  1 1 1 0 1;
		          0 0 0 0 0  1 1 1 1 0];
		labels = [1 1 1 1 2  2 3 3 3 3];
	else
		% LFR benchmark
		[name,rest] = strtok(name);
		if isequal(name,'GN')
			[A,labels] = random_powerlaw_graph(['-N 128 -k 16 -maxk 16 -minc 32 -maxc 32 -mu ',rest]);
		elseif isequal(name,'Small') || isequal(name,'Small1000') || isequal(name,'Small1k')
			[A,labels] = random_powerlaw_graph(['-N 1000 -t1 2 -t2 1 -k 20 -maxk 50 -minc 10 -maxc 50 -mu ',rest]);
		elseif isequal(name,'Big') || isequal(name,'Big1000') || isequal(name,'Big1k')
			[A,labels] = random_powerlaw_graph(['-N 1000 -t1 2 -t2 1 -k 20 -maxk 50 -minc 20 -maxc 100 -mu ',rest]);
		elseif isequal(name,'Small5000') || isequal(name,'Small5k')
			[A,labels] = random_powerlaw_graph(['-N 5000 -t1 2 -t2 1 -k 20 -maxk 50 -minc 10 -maxc 50 -mu ',rest]);
		elseif isequal(name,'Big5000') || isequal(name,'Big5k')
			[A,labels] = random_powerlaw_graph(['-N 5000 -t1 2 -t2 1 -k 20 -maxk 50 -minc 20 -maxc 100 -mu ',rest]);
		else
			error(["Unknown dataset: " name]);
		end
	end
end
