
function [A,c] = random_powerlaw_graph(params, seed)
	% Call Santo Fortunato's LFR graph generator
	%
	% Usage
	%   [A,labels] = random_powerlaw_graph(params, [seed])
	% Parameters
	%   params:  a parameter string to pass to the benchmark graph generator
	%   seed:    the random seed, optional
	
	
	% default parameters
	if nargin < 1, params = ''; end
	
	% Try to locate 'benchmark' program
	windows = any(pwd=='\');
	if windows
		dev_null = 'nul';
	else
		dev_null = '/dev/null';
	end
	exe = file_in_path(path,'benchmark');
	if isempty(exe)
		exe = file_in_path(path,'benchmark.exe');
	end
	if isempty(exe)
		if windows
			exe = [pwd '\\lfr_binary_networks\benchmark'];
		else
			exe = [pwd '/lfr_binary_networks/benchmark'];
		end
	end
	
	% work in a temp directory
	cur_pwd = pwd;
	temp_pwd = tmpnam;
	mkdir(temp_pwd);
	chdir(temp_pwd);
	
	if nargin < 2
		seed = int32(rand*1e9) + time;
	end
	dlmwrite(['time_seed.dat'],seed);
	
	default_params = '-N 1000 -k 32 -maxk 128 -minc 10 -mu 0.1';
	cmd = sprintf('%s %s %s > %s',exe,default_params,params,dev_null);
	system(cmd);
	
	if exist('network.dat') == 2 && exist('community.dat') == 2
		
		net = load('network.dat');
		com = load('community.dat');
		
		A = spconvert([net,ones(size(net,1),1)]);
		c = com(:,2);
		
		% sort for easy visualization
		[c,p] = sort(c);
		A = A(p,p);
	end
	
	% Cleanup
	delete('network.dat');
	delete('community.dat');
	delete('statistics.dat');
	delete('time_seed.dat');
	chdir(cur_pwd);
	rmdir(temp_pwd);
	
	if exist('A') ~= 1
		error('Graph generator failed, command was %s', cmd);
	end
	
end
