function run_real_world_experiments(w,varargin)
	% Run real world experiments with the given trained weight vector
	% Store results in a .tex file
	
	params = struct(varargin{:});
	
	classifier = @(A)find_communities_with_edge_classifier(A,edge_features(A,varargin{:}),w,varargin{:});
	methods = {};
	methods{end+1} = struct('name','Classifier', 'func',classifier);
	methods{end+1} = struct('name','R. Weak',    'func',@(A)radicchi(A,'weak'));
	methods{end+1} = struct('name','R. Strong',  'func',@(A)radicchi(A,'strong'));
	methods{end+1} = struct('name','Infomap',    'func',@infomap);
	
	datasets = {};
	datasets{end+1} = struct('name','zachary',  'internal_name','zachary');
	datasets{end+1} = struct('name','football', 'internal_name','football');
	datasets{end+1} = struct('name','polbooks', 'internal_name','polbooks');
	datasets{end+1} = struct('name','polblogs', 'internal_name','polblogs');
	
	file = fopen(params.filename,'wt');
	fprintf(file,'\\begin{tabular}{l%scc}\n');
	fprintf(file,'\\hline\n');
	fprintf(file,'        & multicolumn{%d}{c}{Normalized Mutual Information} & multicolumn{2}{c}{Number of communties}\\\\', numel(methods));
	fprintf(file,'Dataset & ');
	for im = 1:numel(methods)
		fprintf(file,'%s & ', methods{im}.name);
	end
	fprintf(file,'Actual & Classifier \\\\\n');
	fprintf(file,'\\hline\n');
	
	for id = 1:numel(datasets)
		[A,c] = load_dataset(datasets{id}.internal_name);
		fprintf(file,'%s & ',datasets{id}.name);
		for im = 1:numel(methods)
			func = methods{im}.func;
			d = func(A);
			nmi = normalized_mutual_information(c,d);
			fprintf(file,'%.3f & ',nmi);
			fprintf(stdout, '%s : %s : nmi = %.3f  num = %d\n', datasets{id}.name, methods{im}.name, nmi, numel(unique(d)));
		end
		d_classifier = classifier(A);
		fprintf(file,'%d & %d \\\\\n', numel(unique(c)), numel(unique(d_classifier)));
	end
	fprintf(file,'\\hline\n');
	fprintf(file,'\\end{tabular}\n');
	fclose(file);
end
