function [best_w,best_nmi] = train_svm_on_dataset(tr_A,tr_c,tr_X, va_A,va_c,va_X, varargin)
	% Train an optimal svm on the given dataset.
	% extra parameters are passed to edge_features()
	
	params = struct(varargin{:});
	if ~isfield(params,'learn_method'), params.learn_method = @train_edgewise_svm_liblinear; end
	if ~isfield(params,'tune_mode'), params.tune_mode = 'full'; end
	
	% cross validate to pick best parameters
	if isequal(params.tune_mode,'full')
		lambdas = 10.^[-6:3];
		weights = [10.^[-5:0.5:2]];
		stds = [0,1];
		tune = 1;
	elseif isequal(params.tune_mode,'path')
		lambdas = 10.^[-6:3];
		weights = [10.^[-2:0.5:2]];
		stds = [0];
		tune = 1;
	elseif isequal(params.tune_mode,'fast')
		lambdas = [1];
		weights = [0.01];
		stds = [1];
		tune = 0;
	elseif isequal(params.tune_mode,'fast_path')
		lambdas = [1];
		weights = [1];
		stds = [0];
		tune = 0;
	end
	learn_method = params.learn_method;
	
	best_nmi = -1;
	best_lambda = 0;
	
	i=0;
	for lambda = lambdas
		for ws = weights
			for std = stds;
				params = {'lambda',lambda,'weight_same',ws,'standardize',std};
				w = learn_method(tr_A~=0,tr_X,tr_c,params{:});
				if tune
					va_d = find_communities_with_edge_classifier(va_A,va_X,w, 'min_edge_per_node',1, varargin{:});
					nmi = normalized_mutual_information(va_c,va_d,'max');
					if nmi > best_nmi || (nmi == best_nmi && lambda > best_lambda)
						best_nmi = nmi;
						best_lambda = lambda; % prefer higher lambda
						best_ws  = ws;
						best_std = std;
						best_w   = w;
					end
					% spinner
					printf("%c\b",'/-\\|'(mod(i++,4)+1));fflush(stdout);
				else
					best_nmi = -1;
					best_lambda = lambda;
					best_ws  = ws;
					best_std = std;
					best_w   = w;
				end
			end
		end
	end
	
	if tune
		best_nmi
		best_lambda
		best_std
		best_ws
	end
end
